@@ -1,35 +1,35 @@
 use strict;
 use warnings;
-package Dist::Zilla::Plugin::GenerateFile::ShareDir; # git description: v0.005-37-g6273d30
-# ABSTRACT: Create files in the build, based on a template located in a dist sharedir
+package Dist::Zilla::Plugin::GenerateFile::ShareDir; # git description: v0.006-8-g7955ef5
+# ABSTRACT: Create files in the repository or build, based on a template located in a dist sharedir
 # KEYWORDS: plugin distribution generate create file sharedir template
 # vim: set ts=8 sts=4 sw=4 tw=115 et :
 
-our $VERSION = '0.006';
+our $VERSION = '0.007';
 
 use Moose;
 with (
     'Dist::Zilla::Role::FileGatherer',
     'Dist::Zilla::Role::FileMunger',
     'Dist::Zilla::Role::TextTemplate',
+    'Dist::Zilla::Role::RepoFileInjector',
     'Dist::Zilla::Role::AfterBuild',
     'Dist::Zilla::Role::AfterRelease',
 );
 
-use Carp qw( croak );
 use MooseX::SlurpyConstructor 1.2;
 use Moose::Util 'find_meta';
 use File::ShareDir 'dist_file';
 use Path::Tiny 0.04;
 use Encode;
-use Moose::Util::TypeConstraints qw( enum );
+use Moose::Util::TypeConstraints 'enum';
 use namespace::autoclean;
 
 has dist => (
     is => 'ro', isa => 'Str',
     init_arg => '-dist',
     lazy => 1,
-    default => sub {(my $dist = find_meta(shift)->name) =~ s/::/-/g; $dist },
+    default => sub { (my $dist = find_meta(shift)->name) =~ s/::/-/g; $dist },
 );
 
 has filename => (
@@ -53,27 +53,17 @@ has encoding => (
 );
 
 has location => (
+    is => 'ro', isa => enum([ qw(build root) ]),
+    lazy => 1,
+    default => 'build',
     init_arg => '-location',
-    is       => 'ro',
-    isa      => enum( [qw( build root )] ),
-    lazy     => 1,
-    default  => 'build',
 );
 
 has phase => (
+    is => 'ro', isa => enum([ qw(build release) ]),
+    lazy => 1,
+    default => 'release',
     init_arg => '-phase',
-    is       => 'ro',
-    isa      => enum( [qw( build release )] ),
-    lazy     => 1,
-    default  => 'release',
-);
-
-has _stashed_files => (
-    isa      => 'ArrayRef',
-    init_arg => undef,
-    is       => 'ro',
-    lazy     => 1,
-    default  => sub { [] },
 );
 
 has _extra_args => (
@@ -102,7 +92,7 @@ around dump_config => sub
     my ($orig, $self) = @_;
     my $config = $self->$orig;
 
-    $config->{'' . __PACKAGE__} = {
+    $config->{+__PACKAGE__} = {
         # XXX FIXME - it seems META.* does not like the leading - in field
         # names! something is wrong with the serialization process.
         'dist' => $self->dist,
@@ -111,6 +101,7 @@ around dump_config => sub
         'destination_filename' => $self->filename,
         'location' => $self->location,
         $self->location eq 'root' ? ( 'phase' => $self->phase ) : (),
+        blessed($self) ne __PACKAGE__ ? ( version => $VERSION ) : (),
         $self->_extra_args,
     };
     return $config;
@@ -132,21 +123,27 @@ sub gather_files
         encoding => $self->encoding,    # only used in Dist::Zilla 5.000+
         content => $content,
     );
-    if ( 'build' eq $self->location ) {
-        $self->add_file($file);
-        return;
-    }
 
-    # root eq $self->location
-    push @{ $self->_stashed_files }, $file;
+    if ($self->location eq 'build')
+    {
+        $self->add_file($file);
+    }
+    else
+    {
+        # root eq $self->location
+        $self->add_repo_file($file);
+    }
     return;
 }
 
 around munge_files => sub
 {
-    my ( $orig, $self, @args ) = @_;
-    return $self->$orig(@args) if 'build' eq $self->location;
-    for my $file ( @{ $self->_stashed_files } ) {
+    my ($orig, $self, @args) = @_;
+
+    return $self->$orig(@args) if $self->location eq 'build';
+
+    for my $file ($self->_repo_files)
+    {
         if ($file->can('is_bytes') and $file->is_bytes)
         {
             $self->log_debug([ '%s has \'bytes\' encoding, skipping...', $file->name ]);
@@ -180,41 +177,16 @@ sub munge_file
 
 sub after_build
 {
-    my ($self) = @_;
-    return unless $self->location eq 'root';
-    return unless $self->phase eq 'build';
-    $self->_write_file_root($_) for @{ $self->_stashed_files };
+    my $self = shift;
+    $self->write_repo_files if $self->phase eq 'build';
 }
 
 sub after_release
 {
-    my ($self) = @_;
-    return unless $self->location eq 'root';
-    return unless $self->phase eq 'release';
-    $self->_write_file_root($_) for @{ $self->_stashed_files };
+    my $self = shift;
+    $self->write_repo_files if $self->phase eq 'release';
 }
 
-# That I have to write this clearly says something is wrong.
-# Appropriated from Dist::Zilla::_write_out_file and then made to work with root
-sub _write_file_root
-{
-    my ( $self, $file ) = @_;
-
-    $self->log_debug([ 'writing out %s', $file->name ]);
-
-    my $file_path = path( $file->name );
-    my $to        = path( $self->zilla->root )->child($file_path);
-    my $to_dir    = $to->parent;
-    $to_dir->mkpath unless -e $to_dir;
-
-    croak "not a directory: $to_dir" unless -d $to_dir;
-
-    $self->log_debug([ 'Overwriting %s', $to ]);
-    # we already encoded the content in munge_file if on older Dist::Zilla
-    $to->spew_raw($file->can('encoded_content') ? $file->encoded_content : $file->content);
-
-    chmod $file->mode, "$to" or croak "couldn't chmod $to: $!";
-}
 __PACKAGE__->meta->make_immutable;
 
 __END__
@@ -227,11 +199,11 @@ __END__
 
 =head1 NAME
 
-Dist::Zilla::Plugin::GenerateFile::ShareDir - Create files in the build, based on a template located in a dist sharedir
+Dist::Zilla::Plugin::GenerateFile::ShareDir - Create files in the repository or build, based on a template located in a dist sharedir
 
 =head1 VERSION
 
-version 0.006
+version 0.007
 
 =head1 SYNOPSIS
 
@@ -302,14 +274,6 @@ Only relevant when C<-location = root>. When C<build> (the default), the file
 is written on every build operation. When C<release>, it is only written after
 the distribution is released.
 
-=head1 SUPPORT
-
-=for stopwords irc
-
-Bugs may be submitted through L<the RT bug tracker|https://rt.cpan.org/Public/Dist/Display.html?Name=Dist-Zilla-Plugin-GenerateFile-ShareDir>
-(or L<bug-Dist-Zilla-Plugin-GenerateFile-ShareDir@rt.cpan.org|mailto:bug-Dist-Zilla-Plugin-GenerateFile-ShareDir@rt.cpan.org>).
-I am also usually active on irc, as 'ether' at C<irc.perl.org>.
-
 =head1 SEE ALSO
 
 =for stopwords templated
@@ -338,6 +302,19 @@ L<[GenerateFile]|Dist::Zilla::Plugin::GenerateFile> - generate a (possibly-templ
 
 =back
 
+=head1 SUPPORT
+
+Bugs may be submitted through L<the RT bug tracker|https://rt.cpan.org/Public/Dist/Display.html?Name=Dist-Zilla-Plugin-GenerateFile-ShareDir>
+(or L<bug-Dist-Zilla-Plugin-GenerateFile-ShareDir@rt.cpan.org|mailto:bug-Dist-Zilla-Plugin-GenerateFile-ShareDir@rt.cpan.org>).
+
+There is also a mailing list available for users of this distribution, at
+L<http://dzil.org/#mailing-list>.
+
+There is also an irc channel available for users of this distribution, at
+irc://irc.perl.org/#distzilla.
+
+I am also usually active on irc, as 'ether' at C<irc.perl.org>.
+
 =head1 AUTHOR
 
 Karen Etheridge <ether@cpan.org>
@@ -348,7 +325,7 @@ Karen Etheridge <ether@cpan.org>
 
 Kent Fredric <kentfredric@gmail.com>
 
-=head1 COPYRIGHT AND LICENSE
+=head1 COPYRIGHT AND LICENCE
 
 This software is copyright (c) 2013 by Karen Etheridge.
 
